package com.example.survey.util;


import org.apache.commons.io.FileUtils;
import org.apache.poi.ooxml.POIXMLDocument;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.xwpf.usermodel.Document;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.apache.poi.xwpf.usermodel.XWPFTableRow;
import org.apache.xmlbeans.XmlCursor;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.util.*;
import java.util.Map.Entry;

/**
 * @author Pope
 */
// @Component
public class WordUtil {

    private CustomXWPFDocument document;
    private HashMap<String, Method> key2methods;
    private Map<String, Object> values;
    private String outputPath;
    private File temp;

    public WordUtil(Map<String, Object> values, String templatePath, String outputPath)
            throws FileNotFoundException, IOException, NoSuchMethodException, SecurityException {
        temp = new File(UUID.randomUUID() +  "t.docx");
        this.outputPath = outputPath;
        FileUtils.copyFile(new File(templatePath), temp);
        OPCPackage pack = POIXMLDocument.openPackage(temp.getAbsolutePath());
        document = new CustomXWPFDocument(pack);
        this.values = values;

        key2methods = new HashMap<>();

        key2methods.put("$", getClass().getDeclaredMethod("processText", XWPFTableCell.class, String.class));
        key2methods.put("#", getClass().getDeclaredMethod("processImage", XWPFTableCell.class, String.class));
        key2methods.put("@", getClass().getDeclaredMethod("processTable", XWPFTableCell.class, String.class));
    }

    public void export2word()
            throws FileNotFoundException, IOException, NoSuchMethodException, SecurityException {

        List<XWPFTable> tables = document.getTables();
        HashMap<String, XWPFTableCell> runs = new HashMap<>();
        for (XWPFTable table : tables) {
            // 遍历表格
            List<XWPFTableRow> rows = table.getRows();
            for (int i = 0; i < rows.size(); i++) {
                XWPFTableRow row = table.getRow(i);
                List<XWPFTableCell> cells = row.getTableCells();
                for (int j = 0; j < cells.size(); j++) {
                    XWPFTableCell cell = cells.get(j);
                    String key = cell.getText();
                    if (key.length() == 0) {
                        continue;
                    }
                    String keyHead = key.substring(0, 1);
                    String trueKey = key.substring(1);
                    if (key2methods.containsKey(keyHead)) {
                        cell.removeParagraph(0);
                    } else {
                        continue;
                    }
                    if (values.containsKey(trueKey)) {
                        runs.put(key, cell);
                    } else {
                        cell.setText("");
                    }
                }
            }
        }

        for (Entry<String, XWPFTableCell> entry : runs.entrySet()) {
            String key = entry.getKey();
            XWPFTableCell cell = entry.getValue();
            Method method = key2methods.get(key.subSequence(0, 1));
            String trueKey = key.substring(1);
            try {
                method.invoke(this, cell, values.get(trueKey));
                // System.out.println(key + ":" + values.get(key));
            } catch (Exception e) {
                e.printStackTrace();
                cell.setText("");
            }
        }
        OutputStream outputStream = new FileOutputStream(outputPath);
        // document.
        document.write(outputStream);
        outputStream.close();
        document.close();
        FileUtils.forceDelete(temp);
        // XWPFTemplate template = XWPFTemplate.compile(templatePath,
        // configureBuilder.build()).render(values);
        // try (BufferedOutputStream os = new
        // BufferedOutputStream(response.getOutputStream())) {
        // template.write(os);
        // } catch (IOException e) {
        // e.printStackTrace();
        // } finally {
        // try {
        // template.close();
        // } catch (IOException e) {
        // e.printStackTrace();
        // }
        // }

    }

    
    private void processText(XWPFTableCell cell, String msg) {
        // 执行\n换行
        if (msg == null) {
            cell.setText("");
            return;
        }
        String[] st = msg.split("\n");
        for (int p = 0; p < st.length; p++) {
            cell.addParagraph();
        }
        List<XWPFParagraph> pglist = cell.getParagraphs();
        for (int pi = 0; pi < pglist.size(); pi++) {
            XWPFRun run = pglist.get(pi).createRun();
            if(pi >= st.length){
                System.out.println("ERROR!!!!!!!!!!!");
                continue;
            }
            run.setText(st[pi]);
        }

    }

    private void processImage(XWPFTableCell cell, String imgInfo) {
        String[] infos = imgInfo.split(":");
        String imgPath = infos[0];

        int imgHeight = Integer.parseInt(infos[1]);
        int imgWidth = Integer.parseInt(infos[2]);
        XWPFParagraph paragraph = cell.addParagraph();
        int length = paragraph.getRuns().size();
        if (length > 0) {
            for (int idx = (length - 1); idx >= 0; idx--) {
                paragraph.removeRun(idx);
            }
        }
        try {
            String blipId = document.addPictureData(new FileInputStream(new File(imgPath)), Document.PICTURE_TYPE_PNG);
            document.createPicture(blipId, document.getNextPicNameNumber(Document.PICTURE_TYPE_PNG), imgHeight,
                    imgWidth, paragraph);
        } catch (Exception e) {
            try {
                cell.removeParagraph(0);
                processText(cell, infos[3]);
            } catch (Exception e1) {
                cell.setText("");
                e1.printStackTrace();
            }
            // e.printStackTrace();
        }
    }

    private void processTable(XWPFTableCell cell, String tableInfos) {
        String[] infos = tableInfos.split("&");

        String tableMetaInfo = infos[0];
        String tableContent = infos[1];

        String[] mInfos = tableMetaInfo.split(":");
        int rows = Integer.parseInt(mInfos[0]);
        int cols = Integer.parseInt(mInfos[1]);

        XmlCursor cursor = cell.addParagraph().getCTP().newCursor();
        // cursor
        XWPFTable table = cell.insertNewTbl(cursor);

        String[] rowsString = tableContent.split("\r\n");

        for (int i = 0; i < rows; i++) {
            if (rowsString.length <= i) {
                break;
            }

            XWPFTableRow row = table.createRow();

            if (i == 0) {
                for (int j = 0; j < cols; j++) {
                    row.addNewTableCell();
                }
            }

            String[] colsString = rowsString[i].split("\t");

            for (int j = 0; j < cols; j++) {
                if (colsString.length <= j) {
                    break;
                }
                String msg = colsString[j];

                XWPFTableCell tableCell = row.getCell(j);
                tableCell.removeParagraph(0);
                try {
                    processText(tableCell, msg);
                } catch (Exception e) {
                    tableCell.setText("");
                }

            }
        }

    }

    public static String generateImageStr(String path, int w, int h, String alt) {
        return path + ":" + String.valueOf(w) + ":" + String.valueOf(h) + ":" + alt;
    }

}
